/*
Copyright (C) 2001 Charles Samuels <charles@kde.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
  
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.
   
You should have received a copy of the GNU Library General Public License
along with this library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/

#ifndef ITCLIB_H
#define ITCLIB_H

#define async void
#define itcpublic public


namespace ITC
{

class Stub;
class Condition;
class Mutex;
class Thread;


/**
 * the class from which all your implementations should
 * inherit from.  This is Multiple-Inheritence safe.
 *
 * if you use objects with implicit sharing of some sort,
 * (many of the Qt classes for example), using a 
 * detach(), setLength(length()), or such function
 * frequently works, as does the latter on Qt's QString.
 * Since there is nearly no cost in detaching something
 * that doesn't need it, do it whenever it can be
 * concievably a problem:
 *
 * async TestImpl::function(QString str)
 *
 * str.detach();
 * testStub.function(str);
 **/
class Base
{
friend class Stub;

public:
	Base();
	virtual ~Base();

protected:
	/**
	 * the function you call regularly to dispatch new
	 * messages that arrive into your thread
	 **/
	void process(bool block=false);

public:
	Stub *mStub;
};


/**
 * internal, do not use
 **/
class Stub
{
friend class Base;

public:
	Stub(Base *base);
	virtual ~Stub();
	
protected:
	virtual void __itcProcess(const char*)=0;

	void* __readFrom() const;
	void* __readTo() const;
	void __writeTo(void *src) const;
	void __writeFrom(void *src) const;
	
	Mutex *__toFlowLock() const;
	Mutex *__fromFlowLock() const;
	
	Condition *__fromFlowCond() const;
	Condition *__condBlock() const;
	
	int __fromFlowSize() const;
	
	Base *__base() const;
	
protected:
	struct StubPrivate;
	mutable StubPrivate *d;
};

}


#endif
